@@ -22,7 +22,7 @@ void prepare_command(const char *fmt,...);
 
 
 /*
- * Send to server data; a command.
+ * Sends to server data; a command.
  */
 int
 send_command(session *ssn, char *cmd, char *alt)
@@ -47,7 +47,7 @@ send_command(session *ssn, char *cmd, char *alt)
 
 
 /*
- * Prepare data for sending and check that the output buffer size is
+ * Prepares data for sending and check that the output buffer size is
  * sufficient.
  */
 void
@@ -69,7 +69,7 @@ prepare_command(const char *fmt,...)
 
 
 /*
- * Send a response to a command continuation request.
+ * Sends a response to a command continuation request.
  */
 int
 imap_continuation(session *ssn, const char *cont, size_t len)
@@ -84,7 +84,7 @@ imap_continuation(session *ssn, const char *cont, size_t len)
 		return -1;
 
 	if (opts.debug > 0) {
-		int i;
+		unsigned int i;
 
 		debug("sending continuation data (%d):\n\n", ssn->socket);
 		
@@ -139,7 +139,7 @@ imap_logout(session *ssn)
 
 
 /*
- * IMAP STARTTLS: begin TLS negotiation.
+ * IMAP STARTTLS: begins TLS negotiation.
  */
 int
 imap_starttls(session *ssn)
@@ -194,7 +194,7 @@ imap_login(session *ssn, const char *user, const char *pass)
 
 
 /*
- * IMAP SELECT: access a mailbox in READ-WRITE mode.
+ * IMAP SELECT: accesses a mailbox in READ-WRITE mode.
  */
 int
 imap_select(session *ssn, const char *mbox)
@@ -207,7 +207,7 @@ imap_select(session *ssn, const char *mbox)
 
 
 /*
- * IMAP EXAMINE: access a mailbox in READ-ONLY mode.
+ * IMAP EXAMINE: accesses a mailbox in READ-ONLY mode.
  */
 int
 imap_examine(session *ssn, const char *mbox)
@@ -220,7 +220,7 @@ imap_examine(session *ssn, const char *mbox)
 
 
 /*
- * IMAP CREATE: create mailbox.
+ * IMAP CREATE: creates mailbox.
  */
 int
 imap_create(session *ssn, const char *mbox)
@@ -232,6 +232,32 @@ imap_create(session *ssn, const char *mbox)
 }
 
 
+/*
+ * IMAP DELETE: deletes mailbox.
+ */
+int
+imap_delete(session *ssn, const char *mbox)
+{
+
+	prepare_command("%04X DELETE \"%s\"\r\n", tag, mbox);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
+/*
+ * IMAP RENAME: renames mailbox.
+ */
+int
+imap_rename(session *ssn, const char *oldmbox, const char *newmbox)
+{
+
+	prepare_command("%04X RENAME \"%s\" \"%s\"\r\n", tag, oldmbox, newmbox);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
 /*
  * IMAP SUBSCRIBE: adds the specified mailbox name to the server's set of
  * "active" or "subscribed" mailboxes.
@@ -246,6 +272,21 @@ imap_subscribe(session *ssn, const char *mbox)
 }
 
 
+/*
+ * IMAP UNSUBSCRIBE: removes the specified mailbox name to the server's set of
+ * "active" or "subscribed" mailboxes.
+ */
+int
+imap_unsubscribe(session *ssn, const char *mbox)
+{
+
+	prepare_command("%04X UNSUBSCRIBE \"%s\"\r\n", tag, mbox);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
+
 /*
  * IMAP LIST: returns a subset of names from the complete set of all names
  * available.
@@ -304,7 +345,20 @@ imap_append(session *ssn, const char *mbox, const char *flags,
 
 
 /*
- * IMAP CLOSE: delete messages and return to authenticated state.
+ * IMAP CHECK: requests a checkpoint of the currently selected mailbox.
+ */
+int
+imap_check(session *ssn)
+{
+
+	prepare_command("%04X CHECK\r\n", tag);
+
+	return send_command(ssn, obuf.data, NULL);
+}
+
+
+/*
+ * IMAP CLOSE: deletes messages and returns to authenticated state.
  */
 int
 imap_close(session *ssn)
@@ -333,15 +387,15 @@ imap_expunge(session *ssn)
  * IMAP SEARCH: searches the mailbox for messages that match certain criteria.
  */
 int
-imap_search(session *ssn, int uid, const char *charset, const char *criteria)
+imap_search(session *ssn, const char *charset, const char *criteria)
 {
 
 	if (charset != NULL && *charset != '\0')
-		prepare_command("%04X %sSEARCH CHARSET \"%s\" %s\r\n", tag,
-		    (uid ? "UID " : ""), charset, criteria);
+		prepare_command("%04X UID SEARCH CHARSET \"%s\" %s\r\n", tag,
+		    charset, criteria);
 	else
-		prepare_command("%04X %sSEARCH %s\r\n", tag,
-		    (uid ? "UID " : ""), criteria);
+		prepare_command("%04X UID SEARCH %s\r\n", tag,
+		    criteria);
 
 	return send_command(ssn, obuf.data, NULL);
 }
@@ -351,11 +405,10 @@ imap_search(session *ssn, int uid, const char *charset, const char *criteria)
  * IMAP FETCH: retrieves data associated with a message.
  */
 int
-imap_fetch(session *ssn, int uid, const char *mesg, const char *items)
+imap_fetch(session *ssn, const char *mesg, const char *items)
 {
 
-	prepare_command("%04X %sFETCH %s %s\r\n", tag, (uid ? "UID " : ""),
-	    mesg, items);
+	prepare_command("%04X UID FETCH %s %s\r\n", tag, mesg, items);
 
 	return send_command(ssn, obuf.data, NULL);
 }
@@ -365,12 +418,12 @@ imap_fetch(session *ssn, int uid, const char *mesg, const char *items)
  * IMAP STORE: alters data associated with a message.
  */
 int
-imap_store(session *ssn, int uid, const char *mesg, const char *mode,
+imap_store(session *ssn, const char *mesg, const char *mode,
     const char *flags)
 {
 
-	prepare_command("%04X %sSTORE %s %sFLAGS.SILENT (%s)\r\n", tag,
-	    (uid ? "UID " : ""), mesg, (!strncasecmp(mode, "add", 3) ? "+" :
+	prepare_command("%04X UID STORE %s %sFLAGS.SILENT (%s)\r\n", tag,
+	    mesg, (!strncasecmp(mode, "add", 3) ? "+" :
 	    !strncasecmp(mode, "remove", 6) ? "-" : ""), flags);
 
 	return send_command(ssn, obuf.data, NULL);
@@ -381,11 +434,10 @@ imap_store(session *ssn, int uid, const char *mesg, const char *mode,
  * IMAP COPY: copy messages to mailbox.
  */
 int
-imap_copy(session *ssn, int uid, const char *mesg, const char *mbox)
+imap_copy(session *ssn, const char *mesg, const char *mbox)
 {
 
-	prepare_command("%04X %sCOPY %s \"%s\"\r\n", tag, (uid ? "UID " : ""),
-	    mesg, mbox);
+	prepare_command("%04X UID COPY %s \"%s\"\r\n", tag, mesg, mbox);
 
 	return send_command(ssn, obuf.data, NULL);
 }
