@@ -55,6 +55,53 @@ function Mailbox._cached_close(self, account)
 end
 
 
+function Mailbox._send_query(self, criteria, charset)
+    _check_optional(criteria, { 'string', 'table' })
+    _check_optional(charset, 'string')
+
+    if (criteria == nil) then
+        criteria = ''
+    end
+
+    if (charset == nil) then
+        charset = ''
+    end
+
+    if (self._account._login_user(self._account) ~= true) then
+        return
+    end
+
+    if (self._cached_select(self, self._account._imap, self._mailbox) ~=
+        true) then
+        return {}
+    end
+   
+    local query
+    if (type(criteria) == 'string') then
+        query = 'ALL ' .. criteria
+    else 
+        query = _make_query(criteria)
+    end
+
+    local _, results = ifcore.search(self._account._imap, query, charset)
+
+    if (type(options) == 'table' and options.close == true) then
+        self._cached_close(self, self._account._imap)
+    end
+
+    if (results == nil) then
+        return {}
+    end
+
+    local t = {}
+    for n in string.gmatch(results, '%d+') do
+        table.insert(t, tonumber(n))
+    end
+
+    return t
+end
+
+
 function Mailbox._flag_messages(self, mode, flags, messages)
     if (#messages == 0) then
         return
@@ -426,53 +473,11 @@ end
 
 
 function Mailbox.send_query(self, criteria, charset)
-    _check_optional(criteria, { 'string', 'table' })
-    _check_optional(charset, 'string')
-
-    if (criteria == nil) then
-        criteria = ''
-    end
-
-    if (charset == nil) then
-        charset = ''
-    end
-
-    if (self._account._login_user(self._account) ~= true) then
-        return
-    end
-
-    if (self._cached_select(self, self._account._imap, self._mailbox) ~=
-        true) then
-        return {}
-    end
-   
-    local query
-    if (type(criteria) == 'string') then
-        query = 'ALL ' .. criteria
-    else 
-        query = _make_query(criteria)
-    end
-
-    local _, results = ifcore.search(self._account._imap, query, charset)
-
-    if (type(options) == 'table' and options.close == true) then
-        self._cached_close(self, self._account._imap)
-    end
-
-    if (results == nil) then
-        return {}
-    end
-
-    local t = {}
-    for n in string.gmatch(results, '%d+') do
-        table.insert(t, tonumber(n))
-    end
-
-    return t
+    return Set(self._send_query(self, criteria, charset))
 end
 
 function Mailbox.select_all(self)
-    return Set(self.send_query(self))
+    return self.send_query(self)
 end
 
 
@@ -821,116 +826,123 @@ end
 
 
 function Mailbox.is_answered(self)
-    return Set(self.send_query(self, 'ANSWERED'))
+    return self.send_query(self, 'ANSWERED')
 end
 
 function Mailbox.is_deleted(self)
-    return Set(self.send_query(self, 'DELETED'))
+    return self.send_query(self, 'DELETED')
 end
 
 function Mailbox.is_draft(self)
-    return Set(self.send_query(self, 'DRAFT'))
+    return self.send_query(self, 'DRAFT')
 end
 
 function Mailbox.is_flagged(self)
-    return Set(self.send_query(self, 'FLAGGED'))
+    return self.send_query(self, 'FLAGGED')
 end
 
 function Mailbox.is_new(self)
-    return Set(self.send_query(self, 'NEW'))
+    return self.send_query(self, 'NEW')
 end
 
 function Mailbox.is_old(self)
-    return Set(self.send_query(self, 'OLD'))
+    return self.send_query(self, 'OLD')
 end
 
 function Mailbox.is_recent(self)
-    return Set(self.send_query(self, 'RECENT'))
+    return self.send_query(self, 'RECENT')
 end
 
 function Mailbox.is_seen(self)
-    return Set(self.send_query(self, 'SEEN'))
+    return self.send_query(self, 'SEEN')
 end
 
 function Mailbox.is_unanswered(self)
-    return Set(self.send_query(self, 'UNANSWERED'))
+    return self.send_query(self, 'UNANSWERED')
 end
 
 function Mailbox.is_undeleted(self)
-    return Set(self.send_query(self, 'UNDELETED'))
+    return self.send_query(self, 'UNDELETED')
 end
 
 function Mailbox.is_undraft(self)
-    return Set(self.send_query(self, 'UNDRAFT'))
+    return self.send_query(self, 'UNDRAFT')
 end
 
 function Mailbox.is_unflagged(self)
-    return Set(self.send_query(self, 'UNFLAGGED'))
+    return self.send_query(self, 'UNFLAGGED')
 end
 
 function Mailbox.is_unseen(self)
-    return Set(self.send_query(self, 'UNSEEN'))
+    return self.send_query(self, 'UNSEEN')
 end
 
 function Mailbox.is_larger(self, size)
     _check_required(size, 'number')
 
-    return Set(self.send_query(self, 'LARGER ' .. tostring(size)))
+    return self.send_query(self, 'LARGER ' .. tostring(size))
 end
 
 function Mailbox.is_smaller(self, size)
     _check_required(size, 'number')
 
-    return Set(self.send_query(self, 'SMALLER ' .. tostring(size)))
+    return self.send_query(self, 'SMALLER ' .. tostring(size))
 end
 
 
 function Mailbox.arrived_on(self, date)
     _check_required(date, 'string')
 
-    return Set(self.send_query(self, 'ON ' .. date))
+    return self.send_query(self, 'ON ' .. date)
 end
 
 function Mailbox.arrived_before(self, date)
     _check_required(date, 'string')
 
-    return Set(self.send_query(self, 'BEFORE ' .. date))
+    return self.send_query(self, 'BEFORE ' .. date)
 end
 
 function Mailbox.arrived_since(self, date)
     _check_required(date, 'string')
 
-    return Set(self.send_query(self, 'SINCE ' .. date))
+    return self.send_query(self, 'SINCE ' .. date)
 end
 
 function Mailbox.sent_on(self, date)
     _check_required(date, 'string')
 
-    return Set(self.send_query(self, 'SENTON ' .. date))
+    return self.send_query(self, 'SENTON ' .. date)
 end
 
 function Mailbox.sent_before(self, date)
     _check_required(date, 'string')
 
-    return Set(self.send_query(self, 'SENTBEFORE ' .. date))
+    return self.send_query(self, 'SENTBEFORE ' .. date)
 end
 
 function Mailbox.sent_since(self, date)
     _check_required(date, 'string')
 
-    return Set(self.send_query(self, 'SENTSINCE ' .. date))
+    return self.send_query(self, 'SENTSINCE ' .. date)
 end
 
 function Mailbox.is_newer(self, days)
     _check_required(days, 'number')
 
-    return Set(self.send_query(self, 'SINCE ' .. form_date(days)))
+    return self.send_query(self, 'SINCE ' .. form_date(days))
 end
 
 function Mailbox.is_older(self, days)
     _check_required(days, 'number')
 
-    return Set(self.send_query(self, 'BEFORE ' .. form_date(days)))
+    return self.send_query(self, 'BEFORE ' .. form_date(days))
+end
+
+
+function Mailbox.has_flag(self, flag)
+    _check_required(flag, 'string')
+
+    return self.send_query(self, 'KEYWORD "' .. flag .. '"')
 end
 
 
@@ -938,57 +950,56 @@ function Mailbox.contain_field(self, field, string)
     _check_required(field, 'string')
     _check_required(string, 'string')
 
-    return Set(self.send_query(self, 'HEADER ' .. field .. ' "' ..
-        string .. '"'))
+    return self.send_query(self, 'HEADER ' .. field .. ' "' .. string .. '"')
 end
 
 function Mailbox.contain_bcc(self, string)
     _check_required(string, 'string')
 
-    return Set(self.send_query(self, 'BCC "' .. string .. '"'))
+    return self.send_query(self, 'BCC "' .. string .. '"')
 end
 
 function Mailbox.contain_cc(self, string)
     _check_required(string, 'string')
 
-    return Set(self.send_query(self, 'CC "' .. string .. '"'))
+    return self.send_query(self, 'CC "' .. string .. '"')
 end
 
 function Mailbox.contain_from(self, string)
     _check_required(string, 'string')
 
-    return Set(self.send_query(self, 'FROM "' .. string .. '"'))
+    return self.send_query(self, 'FROM "' .. string .. '"')
 end
 
 function Mailbox.contain_subject(self, string)
     _check_required(string, 'string')
 
-    return Set(self.send_query(self, 'SUBJECT "' .. string .. '"'))
+    return self.send_query(self, 'SUBJECT "' .. string .. '"')
 end
 
 function Mailbox.contain_to(self, string)
     _check_required(string, 'string')
 
-    return Set(self.send_query(self, 'TO "' .. string .. '"'))
+    return self.send_query(self, 'TO "' .. string .. '"')
 end
 
 function Mailbox.contain_header(self, string)
     _check_required(string, 'string')
 
-    return Set(self.send_query(self, 'TEXT "' .. string .. '" NOT BODY "' ..
-        string .. '"'))
+    return self.send_query(self, 'TEXT "' .. string .. '" NOT BODY "' ..
+        string .. '"')
 end
 
 function Mailbox.contain_body(self, string)
     _check_required(string, 'string')
 
-    return Set(self.send_query(self, 'BODY "' .. string .. '"'))
+    return self.send_query(self, 'BODY "' .. string .. '"')
 end
 
 function Mailbox.contain_message(self, string)
     _check_required(string, 'string')
 
-    return Set(self.send_query(self, 'TEXT "' .. string .. '"'))
+    return self.send_query(self, 'TEXT "' .. string .. '"')
 end
 
 
@@ -996,7 +1007,7 @@ function Mailbox.match_field(self, field, pattern)
     _check_required(field, 'string')
     _check_required(pattern, 'string')
 
-    local messages = self.send_query(self)
+    local messages = self._send_query(self)
     local mesgs = _make_normal(messages)
     local fields = self._fetch_fields(self, { field }, mesgs)
 
@@ -1047,7 +1058,7 @@ end
 function Mailbox.match_header(self, pattern)
     _check_required(pattern, 'string')
 
-    local messages = self.send_query(self)
+    local messages = self._send_query(self)
     local mesgs = _make_normal(messages)
     local header = self._fetch_header(self, mesgs)
 
@@ -1068,7 +1079,7 @@ end
 function Mailbox.match_body(self, pattern)
     _check_required(pattern, 'string')
 
-    local messages = self.send_query(self)
+    local messages = self._send_query(self)
     local mesgs = _make_normal(messages)
     local body = self._fetch_body(self, mesgs)
 
@@ -1089,7 +1100,7 @@ end
 function Mailbox.match_message(self, pattern)
     _check_required(pattern, 'string')
 
-    local messages = self.send_query(self)
+    local messages = self._send_query(self)
     local mesgs = _make_normal(messages)
     local full = self._fetch_message(self, mesgs)
 
